import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


public class InputManager implements KeyListener {

	private char lastKeyTyped;
	private boolean keyWasTyped;
	private boolean wasbackspace;
	private boolean wasenter;

	private class KeyState
	{
		private static final int STATE_RELEASED = 0;
		private static final int STATE_PRESSED = 1;
		private int state = STATE_RELEASED;
		
		public synchronized void pressed()
		{
			state = STATE_PRESSED;
		}
		public synchronized void released()
		{
			state = STATE_RELEASED;
		}
		
		public synchronized boolean isPressed()
		{
			if (state == STATE_PRESSED)
				return true;
			return false;
		}

	}
	
	private KeyState [] keys = new KeyState[600];
	
	public InputManager(Component comp)
	{
		comp.addKeyListener(this);
		keyWasTyped = false;
		wasenter = false;
		wasbackspace = false;
		keys[KeyEvent.VK_DOWN] = new KeyState();
		keys[KeyEvent.VK_UP] = new KeyState();
		keys[KeyEvent.VK_LEFT] = new KeyState();
		keys[KeyEvent.VK_RIGHT] = new KeyState();
		
		keys[KeyEvent.VK_BACK_SPACE] = new KeyState();
		keys[KeyEvent.VK_DELETE] = new KeyState();
		keys[KeyEvent.VK_ENTER] = new KeyState();
	}

	private KeyState getKey (KeyEvent e)
	{
		int keyCode = e.getKeyCode();
		if (keyCode < keys.length)
			return keys[keyCode];
		return null;
	}
	
    public void keyPressed(KeyEvent e)
    {
    	KeyState ks = getKey(e);
    	if (ks != null)
    		ks.pressed();
    	e.consume();
    }
    public void keyReleased(KeyEvent e) 
    {
    	KeyState ks = getKey(e);
    	if (ks != null)
    		ks.released();
    	e.consume();
    }
    public void keyTyped(KeyEvent e) 
    {
    	if (keys[KeyEvent.VK_BACK_SPACE].isPressed() || keys[KeyEvent.VK_DELETE].isPressed()) {
    		wasbackspace = true;
    	} else if (keys[KeyEvent.VK_ENTER].isPressed()) {
    		wasenter = true;
    	} else  {
    		keyWasTyped = true;
    		lastKeyTyped = e.getKeyChar();
    	}
    	e.consume();
    }
    
    public boolean wasEnter()
    {
    	boolean rtn = wasenter;
    	wasenter = false;
    	return rtn;
    }
    public boolean wasBackSpace()
    {
    	boolean rtn = wasbackspace;
    	wasbackspace = false;
    	return rtn;
    }
    public char getLastKeyTyped()
    {
    	char rtn = '\0';
    	
    	if (keyWasTyped)
    		rtn = lastKeyTyped;
    	keyWasTyped = false;
    	return rtn;
    }
    
    public synchronized boolean isDownPressed()
    {
    	return keys[KeyEvent.VK_DOWN].isPressed();
    }
    public synchronized boolean isUpPressed()
    {
    	return keys[KeyEvent.VK_UP].isPressed();
    }
    public synchronized boolean isLeftPressed()
    {
    	return keys[KeyEvent.VK_LEFT].isPressed();
    }
    public synchronized boolean isRightPressed()
    {
    	return keys[KeyEvent.VK_RIGHT].isPressed();
    }

}
