import java.applet.*;
import java.awt.*;
import java.io.IOException;


public class kworld extends Applet implements Runnable 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private int mapX = 0;
	private int mapY = 0;

	private Image dbImage;
	private Graphics dbg;

    private MediaTracker tracker;
    
    public sprite spriteEngine;
    public tiles worldTiles;
    private InputManager input;
    private createChar createToon;
    
    private Image interfaceImage;
    
    private String chatMsg = "";
    
    private kworldClient client;

	public void init()
	{
		input = new InputManager(this);
		
		client = new kworldClient(this);
		client.hostName = getCodeBase().getHost();

        tracker = new MediaTracker(this);

        Image [] spriteDown  = new Image[8];
        Image [] spriteUp    = new Image[8];
        Image [] spriteLeft  = new Image[8];
        Image [] spriteRight = new Image[8];

        for (int i = 0; i < spriteDown.length; i++) {
        	String raceName = "krabbit";
        	switch(i) {
        		case 0: raceName = "krabbit"; break;
        		case 1: raceName = "spritef"; break;
        		case 2: raceName = "humanf"; break;
        		case 3: raceName = "impf"; break;
        		case 4: raceName = "krabbitm"; break;
        		case 5: raceName = "spritem"; break;
        		case 6: raceName = "humanm"; break;
        		case 7: raceName = "impm"; break;
        	}
        	spriteDown[i] = getImage( getDocumentBase(), "images/characters/"+raceName+"_down.png" );
        	tracker.addImage(spriteDown[i], 0);
        	spriteUp[i] = getImage( getDocumentBase(), "images/characters/"+raceName+"_up.png" );
        	tracker.addImage(spriteUp[i], 0);
        	spriteLeft[i] = getImage( getDocumentBase(), "images/characters/"+raceName+"_left.png" );
        	tracker.addImage(spriteLeft[i], 0);
        	spriteRight[i] = getImage( getDocumentBase(), "images/characters/"+raceName+"_right.png" );
        	tracker.addImage(spriteRight[i], 0);

        }

        
        Image tileImage = getImage( getDocumentBase(), "images/tiles/world.png" );
        tracker.addImage(tileImage, 0);
        Image treeImage = getImage( getDocumentBase(), "images/tiles/treeobj.png" );
        tracker.addImage(treeImage, 0);
        Image signImage = getImage( getDocumentBase(), "images/tiles/sign_post.png" );
        tracker.addImage(signImage, 0);
        
        interfaceImage = getImage( getDocumentBase(), "images/interface/interface.png" );
        tracker.addImage(interfaceImage, 0);

        Image kplanet = getImage( getDocumentBase(), "images/interface/kplanet.png" );
        tracker.addImage(kplanet, 0);
        Image createBGImage = getImage( getDocumentBase(), "images/interface/create.png" );
        tracker.addImage(createBGImage, 0);
        Image createRaceImage = getImage( getDocumentBase(), "images/interface/race_tab.png" );
        tracker.addImage(createRaceImage, 0);
        
        try {
            //Start downloading the images. Wait until they're loaded.
            tracker.waitForAll();
        } catch (InterruptedException e) {}

        
		worldTiles = new tiles(client);

		spriteEngine = new sprite(this, worldTiles);
        for (int i = 0; i < spriteDown.length; i++) {
        	spriteEngine.addSprite(spriteDown[i], spriteDown[i].getWidth(this) / 5, 50, 5);
        	spriteEngine.addSprite(spriteUp[i], spriteUp[i].getWidth(this) / 5, 50, 5);
        	spriteEngine.addSprite(spriteLeft[i], spriteLeft[i].getWidth(this) / 5, 50, 5);
        	spriteEngine.addSprite(spriteRight[i], spriteRight[i].getWidth(this) / 5, 50, 5);
        }

        try {
        	worldTiles.loadMap("maps/world.map");
			worldTiles.setTiles(tileImage, treeImage, signImage, 32, this.getSize().width, this.getSize().height);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		createToon = new createChar(this, this, spriteEngine, client, createBGImage, createRaceImage, kplanet);

		setBackground (Color.black);
	}

	public boolean wasEnter()
	{
		return input.wasEnter();
	}
	public boolean wasBackSpace()
	{
		return input.wasBackSpace();
	}
	public char getLastKeyTyped()
	{
		return input.getLastKeyTyped();
	}
	
	private void sendChat()
	{
		client.sendChat(chatMsg);
		chatMsg = "";
	}
	private void doChat()
	{
		if (wasEnter()) {
			sendChat();
		} else if (wasBackSpace()) {
			int endIndex = chatMsg.length() -1;
			if (endIndex < 0)
				endIndex = 0;
			chatMsg = chatMsg.substring(0, endIndex);
		} else {
			char key = getLastKeyTyped();
			if (key != '\0' && chatMsg.length() < 50)
				chatMsg += key;
		}
	}
	
	public void start ()
	{
		Thread th = new Thread (this);
		th.start ();
	}

	public void stop()
	{
		client.stop();
	}

	public void destroy()
	{

	}

	public void run ()
	{
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);

		while (true)
		{
			repaint();

			try
			{
				Thread.sleep (20);
			}
			catch (InterruptedException ex)
			{
				// do nothing
			}

			Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
		}
	}

	private long lastMoveTime = 0;

	public void update (Graphics g)
	{
		if (dbImage == null)
		{
			dbImage = createImage (this.getSize().width, this.getSize().height);
			dbg = dbImage.getGraphics ();
		}

		boolean isMoving = false;
		int [] outX = new int[1];
		int [] outY = new int[1];
		if (worldTiles.resetMapXY(outX, outY)) {
			mapX = outX[0];
			mapY = outY[0];
			lastMoveTime = System.currentTimeMillis() + 500;
			client.sendLocation(worldTiles.getMapLocation());
			sendMove();
		}

        if (lastMoveTime < System.currentTimeMillis()) {
        	lastMoveTime = System.currentTimeMillis() + 50;
            if (input.isDownPressed()) {
            	spriteEngine.setSpriteIndex(0);
                if (mapY < 100 && !worldTiles.isBlocked(mapX, mapY+1)) {
                    mapY++;
                    isMoving = true;
                }
            } else if (input.isUpPressed()) {
            	spriteEngine.setSpriteIndex(1);
                if (mapY > -10 && !worldTiles.isBlocked(mapX, mapY-1)) {
                    mapY--;
                    isMoving = true;
                }
            }
            
            if (input.isLeftPressed()) {
            	spriteEngine.setSpriteIndex(2);
                if (mapX > -10 && !worldTiles.isBlocked(mapX-1, mapY)) {
                    mapX--;
                    isMoving = true;
                }
            } else if (input.isRightPressed()) {
            	spriteEngine.setSpriteIndex(3);
                if (mapX < 100 && !worldTiles.isBlocked(mapX+1, mapY)) {
                    mapX++;
                    isMoving = true;
                }
            }
        }
        
        if (isMoving)
        	sendMove();
        
        spriteEngine.updateAni(isMoving, System.currentTimeMillis());

		dbg.setColor (getBackground ());
		dbg.fillRect (0, 0, this.getSize().width, this.getSize().height);

		dbg.setColor (getForeground());
		paint (dbg);

		g.drawImage (dbImage, 0, 0, this);
	}

	public void sendMove()
	{
		client.sendMove(createToon.name, createToon.currentRace, spriteEngine.getSDIndex(), spriteEngine.getCurrFame(), mapX, mapY);
	}
	
    private boolean firstPaint = true;
	public void paint (Graphics g)
	{

        if (!tracker.checkAll()) {
            g.clearRect(0, 0, this.getSize().width, this.getSize().height);
            g.drawString("Please wait...", 0, this.getSize().height/2);
        }
        
        if (!createToon.createCharacter(g)) {

        	worldTiles.drawTiles(mapX, mapY, g);
		
        	spriteEngine.drawMainPlayer(worldTiles.getDrawPlayerX() - 5, worldTiles.getDrawPlayerY() + 32, g);
        
        	worldTiles.drawObjs(mapX, mapY, g);
        
        	g.drawImage(interfaceImage, 0 , 0, null);
        	
        	g.drawString("X: " + (worldTiles.getPlayerX()+mapX) + " Y: " + (worldTiles.getPlayerY()+mapY), 525, 30);
        	g.drawString(createToon.name, 20, 55);
        	
        	client.drawChatText(g);
        	
        	doChat();
        	g.drawString(chatMsg, 118, 464);
        }

        if (firstPaint) {
        	requestFocus();
        	firstPaint = false;
        }
        
	}
}

