import javax.net.ssl.*;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.lang.ClassNotFoundException;
import java.util.Enumeration;

public class playerConnect implements Runnable, socketConnection {

	private kworldServer server;
	private SSLSocket socket;
	public String Name;
	public String ip;
	public int location;
	
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private boolean finalized;
	
	private messageQueue queue;
	
	private int Race;
	private int SDIndex;
	private int X;
	private int Y;
	private int AniFrame;
	
	playerConnect(kworldServer kws, SSLSocket s)
	{
		try {
			Name = null;
			location = 0;
			finalized = false;
			server = kws;
			socket = s;
			Race = 0;
			SDIndex = 0;
			AniFrame = 0;
			X = 0;
			Y = 0;
			
			ip = s.getInetAddress().getHostName();
			server.Log("IP: " + ip);

			out = new ObjectOutputStream(s.getOutputStream());
			in = new ObjectInputStream(s.getInputStream());
			
			queue = new messageQueue(this);
			
			new Thread(this).start();
			
		} catch (IOException e) {
			System.out.println("playerConnect error: " + e);
		}
	}
	
	private socketData readObject()
	{
		try {
			return (socketData) in.readObject();
		} catch (IOException e) {
			return null;
		} catch (ClassNotFoundException e) {
			return null;
		}
	}
	
	public void writeObject(Object obj) {
		queue.addTo(obj);
	}
	public void _writeObject(Object obj)
	{
		try {
			out.writeObject(obj);
			out.flush();
		} catch (IOException e) {
			close();
		}
	}
	
	public void close()
	{
		server.kill(this);
		try {
			socket.close();
		} catch (IOException e) { e.printStackTrace(); }
	}
	
// =======================================================
	public void playerAdd(String playername)
	{
		if (playername == null) {
			writeObject(new SD_Error("no username recieved"));
			return;
		}
		if (Name == null && !finalized) {
			Name = playername;
			server.Log("playerAdd: " + Name);
			finalized = server.finalizePlayer(playername, this);
			if (!finalized) {
				Name = null;
				writeObject(new SD_Kick(null));
			}
		} else {
			writeObject(new SD_Error("already recieved name: " + Name));
		}
	}
	
	public void playerDel(String playername)
	{
		close();
	}
	
	@SuppressWarnings("static-access")
	public void kick(String playername)
	{
		playerConnect pc = (playerConnect) server.connectedPlayers.get(playername);
		if (pc == null) {
			writeObject(new SD_Error("no playername " + playername));
		} else {
			server.sendTo(new SD_Error("Bye Bye"), playername);
			pc.close();
		}
	}
	
	@SuppressWarnings("static-access")
	public void location(int locationNum)
	{
		if (server.locations.validLocation(locationNum)) {
			if (location == locationNum) {
				writeObject(new SD_Error("Already in location " + locationNum));
			} else {
				//server.Log("location: " + Name + " oldLoc" + location + " newLoc:" + locationNum);
				server.broadcast(new SD_PlayerDel(Name), Name, location);
				server.locations.delPlayer(location);
				location = locationNum;
				//server.broadcast(new SD_Location(location), null);
				//server.broadcast(new SD_PlayerAdd(Name), Name, locationNum);
				writeObject(new SD_Location(locationNum));
				server.sendPlayerList(this);
			}
		} else {
			writeObject(new SD_Error("location " + locationNum  + " does not exist"));
		}
	}
	
	public void move(String name, int race, int index, int frame, int x, int y)
	{
		if (finalized) {
			Race = race;
			X = x;
			Y = y;
			SDIndex = index;
			AniFrame = frame;
			//System.out.println("name:"+ name + " race:"+race + " index:"+index + " frame:" + frame + " X:"+x + " y:" + y);
			server.broadcast(new SD_Move(name, race, index, frame, x, y), name, location);
		} else {
			writeObject(new SD_Error("connection not confirmed"));
		}
	}
	
	public int getX()
	{
		return X;
	}
	public int getY()
	{
		return Y;
	}
	public int getRace()
	{
		return Race;
	}
	public int getSDIndex()
	{
		return SDIndex;
	}
	public int getAniFrame()
	{
		return AniFrame;
	}
	
	@SuppressWarnings("static-access")
	public void who(String playername)
	{
		if (finalized) {
			String message = "";
			Enumeration<?> e = server.connectedPlayers.keys();
			while (e.hasMoreElements()) {
				String tmpName = (String) e.nextElement();
				playerConnect o = (playerConnect) server.connectedPlayers.get(tmpName);
				message += (o.Name + ", ");
				if (message.length() >= 50) {
					writeObject(new SD_Chat("Who", message));
					message = "";
				}
			}
			if (message.length() > 0)
				writeObject(new SD_Chat("Who", message));
		} else {
			writeObject(new SD_Error("connection not confirmed"));
		}
	}
	
	public void chat(String playername, String message)
	{
		if (finalized) {
			server.broadcast(new SD_Chat(Name, message), Name, location);
		} else {
			writeObject(new SD_Error("connection not confirmed"));
		}
	}
	
	public void private_msg(String playername, String message)
	{
		if (finalized) {
			if (!server.sendTo(new SD_Private(Name, message), playername)) {
				writeObject(new SD_Error("No player named " + playername + " found."));
			}
		} else {
			writeObject(new SD_Error("connection not confirmed"));
		}
	}

	public void whisper(String playername, String message)
	{
		if (finalized) {
			if (!server.sendTo(new SD_Whisper(Name, message), playername)) {
				writeObject(new SD_Error("No player named " + playername + " found."));
			}
		} else {
			writeObject(new SD_Error("connection not confirmed"));
		}
	}
	
	public void error(String err)
	{
		System.out.println(err);
	}
// =======================================================
	public void run()
	{
		socketData sd;
		while ((sd = readObject()) != null) {
			sd.performAction(this);
		}
		close();
	}
}
